{{
  Jonathan Dummer
  Half-Duplex Serial
}}

CON
  buf_mask = $FF
  buf_len = 256

VAR
  ' for interfacing with the PASM cog
  long read_buf_ptr
  long write_buf_ptr
  ' holder for transmitting a character or hex value
  long send_temp
  ' for tracking the data
  long rx_head          
  long rx_tail
  byte rx_buffer[buf_len]
  ' for tracking the cog ID
  byte cog
  

PUB start( rx_pin, tx_pin, rate )
  ' start by stopping
  stop
  ' set up the pins
  maskRX := |< rx_pin
  maskTX := |< tx_pin
  ' how many clocks between bits
  bit_clocks := (clkfreq + (rate>>1)) / rate
  ' from the edge of the start bit to the middle of the 1st data bit
  ' (I wanted to adjust for ~16..28 clocks from the test and the
  ' intervening commands, but it works better allowing the lag, as
  ' long as the lag is < 1/2 the bit_clocks...which it is at 1Mbps)
  initial_clocks := bit_clocks + (bit_clocks >> 1)
  ' set up the buffers and pointers
  bytefill( @rx_buffer, 0, buf_len )
  buf_beg := @rx_buffer
  rx_head~
  rx_tail~
  read_buf_ptr := @rx_buffer
  write_buf_ptr~
  rx_head_ptr := @rx_head
  ' start the new cog
  cog := cognew( @HDS_entry, @write_buf_ptr ) + 1
  return cog
  
PUB stop
  if cog
    cogstop( cog~ - 1 )

PUB str( string_ptr )
  repeat while write_buf_ptr
  write_buf_ptr := (string_ptr << 16) | strsize( string_ptr )  

PUB tx( char_val )  
  repeat while write_buf_ptr
  send_temp := char_val  
  write_buf_ptr := (@send_temp << 16) | 1

PUB hex( value, dummy_length )
  repeat while write_buf_ptr
  send_temp := value  
  write_buf_ptr := (@send_temp << 16) 

'' Lifted from Full-Duplex Serial
PUB rxflush
'' Flush receive buffer
  repeat while rxcheck => 0
    
PUB rxcheck : rxbyte
'' Check if byte received (never waits)
'' returns -1 if no byte received, $00..$FF if byte
  rxbyte--
  if rx_tail <> rx_head
    rxbyte := rx_buffer[rx_tail++]
    rx_tail &= buf_mask

PUB rxtime(ms) : rxbyte | t
'' Wait ~ms milliseconds for a byte to be received
'' returns -1 if no byte received, $00..$FF if byte
  t := (clkfreq >> 10) * ms + cnt
  repeat until (rxbyte := rxcheck) => 0 or (cnt - t) > 0

PUB rx : rxbyte
'' Receive byte (may wait for byte)
'' returns $00..$FF
  repeat while (rxbyte := rxcheck) < 0

PUB dec(value) | i
  ' Print a decimal number
  if value < 0
    -value
    tx("-")
  i := 1_000_000_000
  repeat 10
    if value => i
      tx(value / i + "0")
      value //= i
      result~~
    elseif result or i == 1
      tx("0")
    i /= 10 

{
PUB hex(value, digits)
  ' Print a hexadecimal number
  value <<= (8 - digits) << 2
  repeat digits
    tx(lookupz((value <-= 4) & $F : "0".."9", "A".."F"))
'}

{ serial PASM code }  
DAT
ORG 0

HDS_entry     ' setup
              mov outa,maskTX
              mov dira,maskTX

              ' keep checking for a pointer or a start bit
main_loop     rdlong write_ptr,par wz
              test maskRX,ina wc
if_c_and_z    jmp #main_loop
              ' so, was it a start bit?
if_nc         jmp #read_byte
              ' otherwise we are going to send some data
              ' (data to send can just wait for another byte)
              mov bytes_left,write_ptr
              shr write_ptr,#16
              and bytes_left,#511 wz
if_z          jmp #send_out_hex
get_byte              
              rdbyte data,write_ptr
              add write_ptr,#1
              ' send it
              call #write_byte
              ' do I have more bytes to write?
              djnz bytes_left,#get_byte
              
done_writing  ' signal that I'm done with the date
              mov write_ptr,#0
              wrlong write_ptr,par
              ' start over
              jmp #main_loop

send_out_hex
              ' get the value, and then signal that I have it
              rdlong hex_value,write_ptr
              mov write_ptr,#0
              mov bytes_left,#8
              wrlong write_ptr,par
              mov sent_1,#0
              ' now send out the (up to) 8 bytes              
:hex_byte
              ' make sure to send on the last byte
              cmp bytes_left,#1 wz
if_z          mov sent_1,#1
              ' get the next highest 4 bits into position
              rol hex_value,#4
              mov data,hex_value
              and data,#%1111 wz
              ' if this nibble is 0, we may not need to send it
if_z          tjz sent_1,#:done_with_hex_byte
              ' now check if we're in the 0..9 range, or the A..F range
              cmp data,#10 wz,wc
if_ae         add data,#("A" - 10)
if_b          add data,#"0"
              ' send it
              call #write_byte
              mov sent_1,#1
:done_with_hex_byte
              ' go back for more
              djnz bytes_left,#:hex_byte

              ' start over
              jmp #main_loop
              

write_byte
              mov tmp,#8              
              mov timestamp,bit_clocks
              add timestamp,cnt
              andn outa,maskTX
:write_bit    shr data,#1 wc
              waitcnt timestamp,bit_clocks
              muxc outa,maskTX
              djnz tmp,#:write_bit
              waitcnt timestamp,bit_clocks
              or outa,maskTX
              waitcnt timestamp,bit_clocks
write_byte_ret
              ret


read_byte     ' gentlemen, synchronize your watches
              mov timestamp,initial_clocks
              add timestamp,cnt              
              mov tmp,#8
              mov data,#0
              ' read in the byte
:read_bit     waitcnt timestamp,bit_clocks
              test maskRX,ina wc
              rcl data,#1
              djnz tmp,#:read_bit
              ' where do I save this byte?
              rdlong buf_ptr,rx_head_ptr
              add buf_ptr,buf_beg
              ' of course, the byte is backward
              rev data,#24
              ' save this byte
              wrbyte data,buf_ptr
              sub buf_ptr,buf_beg
              add buf_ptr,#1
              and buf_ptr,#buf_mask              
              ' wait for another bit
              waitcnt timestamp,bit_clocks
              ' update the head
              wrlong buf_ptr,rx_head_ptr
              ' do I need to reset?
              shl history_bytes,#8
              or history_bytes,data
              cmp history_bytes,reset_trigger wz
if_z          jmp #reset_the_prop
              ' start over
              jmp #main_loop


reset_the_prop
              mov tmp,#%1000_0000
              clkset tmp
              
        
{-------- ASM parameters --------}
bit_clocks              long 0
initial_clocks          long 0
maskRX                  long 0
maskTX                  long 0
buf_beg                 long 0
rx_head_ptr             long 0

history_bytes           long 0
reset_trigger           byte "T","S","R","*" ' Little Endian

{-------- ASM variables --------}
tmp           res 1
data          res 1
bytes_left    res 1
timestamp     res 1
buf_ptr       res 1
write_ptr     res 1
hex_value     res 1
sent_1        res 1

FIT 497